from tkinter import *
from tkinter import messagebox
from tkinter import filedialog
import string
MAXBLOCK = 65536
SAFEBLOCK = 1024*1024
def little16(c): # 2.7ではstringだったがバイナリ・ファイルの入力はバイトのリストになった
    return c[0] + (c[1]<<8)
def little32(c):
    return c[0] | (c[1]<<8) | (c[2]<<16) | (c[3]<<24)
def safe_read(fp, size):
    if size <= 0:
        return ""
    if size <= SAFEBLOCK:
        return fp.read(size)
    data = []
    while size > 0:
        block = fp.read(min(size, SAFEBLOCK))
        if not block:
            break
        for i in range(len(block)): # 2.7ではstringとして扱っていたので
            data.append(block[i])   # data.append(block)だけで良かった。
        size = size - len(block)
    return data
HexaTab=('0','1','2','3','4','5','6','7','8','9','A', 'B', 'C', 'D', 'E', 'F')
def hexa2(A):
    B=int(A) & 0xFF
    return HexaTab[int(B/16)]+HexaTab[B % 16]
def RGB(R,G,B):
    return "#" + hexa2(R)+ hexa2(G)+ hexa2(B)

class DIBfileHeader:
    def __init__(self):
        self.bfType=0x4D42
    def read(self, fp):
        fp.seek(0)
        s=fp.read(2)
        self.Type=little16(s)
        #fp.seek(2)
        #print("ファイル種類＝",hex(self.Type))
        s=fp.read(4)
        print(hex(s[0]),hex(s[1]),hex(s[2]),hex(s[3]))
        self.Size=little32(s)
        #print("ファイル全体のサイズ＝",self.Size)
        s=fp.read(4)
        s=fp.read(4)
        self.OffsetBits=little32(s)
        #print("ピクセルビットオフセット＝",self.OffsetBits)
    def print(self):
        print("            ファイル種類： ",hex(self.Type),"(",self.Type,")")
        print("    ファイル全体のサイズ： ",hex(self.Size),"(",self.Size,")")
        print("ピクセルビットオフセット： ",hex(self.OffsetBits),"(",self.OffsetBits,")")
class BitmapInfo:
    def __init__(self):
        self.bcSize=0
    def read(self,fp):
        fp.seek(14)
        s=fp.read(4)
        self.Size=little32(s)
        self.RGBsize=4
        #dt=safe_read(fp,self.Size-4)
        if self.Size==12: # IBM BMP
            self.RGBsize=3
            s=fp.read(2)
            self.Width=little16(s)
            s=fp.read(2)
            self.Height=little16(s)
            s=fp.read(2)
            self.Planes=1
            s=fp.read(2)
            self.BitCount=little16(s)
        else:            # Microaoft Bmp
            s=fp.read(4)
            self.Width=little32(s)
            s=fp.read(4)
            self.Height=little32(s)
            s=fp.read(2)
            self.Planes=1
            s=fp.read(2)
            self.BitCount=little16(s)
            s=fp.read(4)
            self.Compression=little32(s)
            s=fp.read(4)
            self.SizeImage=little32(s)
            s=fp.read(4)
            self.XPperMeter=little32(s)
            s=fp.read(4)
            self.YPperMeter=little32(s)
            s=fp.read(4)
            self.ColorUsed=little32(s)
            s=fp.read(4)
            self.ColorImportant=little32(s)
    def print(self):
        print("                    Size : ", hex(self.Size          ),"(",self.Size,")")
        print("                   Width : ", hex(self.Width         ),"(",self.Width,")")
        print("                  height : ", hex(self.Height        ),"(",self.Height,")")
        print("               Bit Count : ", hex(self.BitCount      ),"(",self.BitCount,")")
        print("             Compression : ", hex(self.Compression   ),"(",self.Compression,")")
        print("             Image Bytes : ", hex(self.SizeImage     ),"(",self.SizeImage,")")
        print("           X pixel/Meter : ", hex(self.XPperMeter    ),"(",self.XPperMeter,")")
        print("           Y pixel/Meter : ", hex(self.YPperMeter    ),"(",self.YPperMeter,")")
        print("   Number of Color Table : ", hex(self.ColorUsed     ),"(",self.ColorUsed,")")
        print("         Important Color : ", hex(self.ColorImportant),"(",self.ColorImportant,")")
class Bitmap:
    def __init__(self, header,info):
        self.header=header
        self.info=info
    def read(self,fp):
        if self.info.BitCount>=24:
            self.ColormapLen=0
            self.Colormap=[]
        else:
            self.ColormapLen=self.info.RGBsize*(1<<self.info.BitCount)
            self.Colormap=fp.read(self.ColormapLen)
        self.Len=self.header.Size-self.header.OffsetBits
        self.Buffer=safe_read(fp,self.Len)
def dsplayColorMap(canvas,pClr,N, RGBsize):
    k=0
    Y=-20
    for i in range(N):
        if i% 16==0:
            Y+=30
            X=10
        CL=RGB(pClr[k+2],pClr[k+1],pClr[k])
        canvas.create_rectangle(X,Y,X+20,Y+20, fill=CL,outline='black')
        X+=30
        k+=RGBsize
    root.update()    
root =Tk()
FT=[('BMPファイルlファイル','*.txt;*.*')]
DR='c:\\'
filename=filedialog.askopenfilename(filetypes=FT)
if filename!="":
    try:
        fp=open(filename,'rb')
        fp.seek(0)
        #ファイル・ヘッダ読込み
        header=DIBfileHeader()
        header.read(fp)
        header.print()
        #ファイル・ヘッダ読込み
        info=BitmapInfo()
        info.read(fp)
        info.print()
        root.title("カラー・マップ確認　1画素当たりのビット数 = "+str(info.BitCount))
        #カラーテーブル取り出し
        bitmap=Bitmap(header,info)
        bitmap.read(fp)
        canvas=Canvas(root,width=500,height=500)
        canvas.pack()
 
        if info.BitCount==24:
            messagebox.showerror("カラー・マップ確認","24ビット画像にはカラー・テーブルはありません")
        else:
            dsplayColorMap(canvas,bitmap.Colormap,1<<info.BitCount,info.RGBsize)
        fp.close()
        root.mainloop()
    except IOError:
        raise IOError("Unspported BMP header type)" )
